
# q-expansion of newform 1920.2.a.e, downloaded from the LMFDB on 03 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1920
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-1], [-1], [2], [-2], [-2], [-2], [2], [2], [-6], [4], [2], [-10], [-8], [2], [6], [2], [-10], [8], [-8], [-6], [16], [-12], [-10], [-6], [-6], [6], [-12], [-10], [-2], [-2], [-14], [-18], [-2], [18], [-16], [-14], [-12], [-2], [18], [-10], [-18], [-12], [26], [14], [4], [22], [18], [0], [6], [6], [-16], [18], [22], [-18], [26], [-6], [-12], [18], [-14], [0], [26], [32], [-8], [26], [-2], [-14], [18], [28], [22], [6], [-36], [-10], [-6], [30], [-10], [2], [-18], [-18], [6], [-18], [-10], [-20], [34], [-4], [0], [26], [10], [18], [26], [0], [-8], [-22], [22], [34], [-6], [-6], [-30], [-40], [38], [-8], [-14], [20], [6], [34], [-46], [-4], [-26], [28], [-22], [22], [6], [38], [-10], [-16], [18], [-12], [-2], [38], [-2], [-42], [-22], [14], [-48], [-10], [42], [6], [16], [-22], [-22], [34], [42], [-4], [-14], [-30], [-26], [-42], [8], [-34], [42], [-38], [-46], [-34], [12], [-10], [4], [22], [14], [14], [54], [-46], [-2], [-12], [6], [-20], [36], [52], [-6], [-14], [-22], [-24], [-18], [42], [30], [-18], [50], [20], [-26], [-26], [6], [26], [-2], [32], [2], [56], [6], [-30], [-6], [-26], [46], [6], [-38], [-58], [38], [-34], [-22], [2], [-60], [6], [60], [-38], [24], [-10], [2], [24], [-10], [62], [-34], [6], [66], [-22], [-28], [38], [-62], [-30], [-30], [-48], [20], [22], [-30], [18], [18], [-58], [68], [-44], [-66], [-2], [30], [-42], [-42], [-50], [68], [-18], [-22], [-32], [38], [14], [0], [-54], [-42], [14], [2], [-68], [18], [16], [66], [50], [6], [18], [-32], [52], [-22], [46], [46], [6], [4], [6], [-70], [78], [-42], [30], [50], [62], [-70], [-46], [30], [22], [-4], [58], [-14], [-54], [16], [14], [-38], [78], [-14], [-22], [6], [56], [26], [-10], [8], [-62], [64], [58], [-58], [-28], [-10], [42], [-14], [-42], [16], [-18], [70], [28], [52], [34], [-18], [84], [46], [-54], [-16], [-42], [38], [-38], [18], [-12], [74], [-62], [-32], [-14], [-14], [32], [44], [-38], [82], [28], [-34], [36], [-22], [6], [26], [30], [-4], [-82], [-74], [-42], [-4], [-30], [30], [-90], [50], [42], [18], [-66], [-14], [-46], [8], [42], [6], [-10], [2], [-72], [60], [-22], [-52], [14], [-34], [10], [-42], [-54], [-42], [-30], [40], [-30], [70], [14], [20], [-28], [-2], [22], [2], [34], [-62], [-34], [-66], [24], [70], [-58], [-94], [6], [6], [18], [-6], [-32], [-14], [30], [62], [62], [-94], [-2], [-98], [82], [80], [-66], [-74], [92], [-14], [-22], [-14], [-70], [38], [74], [62], [2], [42], [-76], [-62], [-56], [-70], [-42], [-70], [-6], [-24], [-80], [26], [-24], [-54], [74], [66], [-50], [-18], [-50], [-34], [-94], [56], [78], [-50], [-76], [-90], [26], [-86], [-88], [-10], [10], [18], [-16], [98], [78], [74], [26], [-74], [-6], [-22], [-94], [18], [12], [-6], [-14], [100]]
